/*
	$Id: Sanctuary.java 170 2006-04-08 21:16:55Z syl $

	Copyright (C) 2006 Syllabub
	Maroontress Fast Software.
*/

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.io.BufferedWriter;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import java.util.List;

/**
   Sanctuaryεư饹Ǥ
*/
public final class Sanctuary {

    /** ХåեΥ */
    private static final int BUFFER_SIZE = 4096;

    /**
       ư饹ޤ
    */
    private Sanctuary() {
    }

    /**
       ѥȥ󥳡ǥ󥰤ꤷƥ饤ޤ

       @param dirName ǥ쥯ȥ̾
       @param fileName ե̾
       @param encoding ϥ󥳡ǥ
       @return 饤
    */
    public static PrintWriter createPrintWriter(final String dirName,
						final String fileName,
						final String encoding) {
	FileOutputStream out;
	Writer wr;
	File file;

	if (dirName != null) {
	    File dir = new File(dirName);
	    dir.mkdirs();
	    file = new File(dir, fileName);
	} else {
	    file = new File(fileName);
	}
	try {
	    out = new FileOutputStream(file);
	} catch (IOException e) {
	    e.printStackTrace();
	    System.exit(1);
	    return null;
	}
	try {
	    wr = new OutputStreamWriter(out, encoding);
	} catch (UnsupportedEncodingException e) {
	    file.delete();
	    System.exit(1);
	    return null;
	}
	return new PrintWriter(new BufferedWriter(wr));
    }

    /**
       jarեΤʤΥե򥫥ȥǥ쥯ȥ˽Ϥޤ

       @param s ե̾
    */
    public static void copyFile(final String s) {
	InputStream in = Sanctuary.class.getResourceAsStream(s);
	File file = new File(s);
	try {
	    FileOutputStream out = new FileOutputStream(file);
	    byte[] data = new byte[BUFFER_SIZE];
	    int size;
	    while ((size = in.read(data)) > 0) {
		out.write(data, 0, size);
	    }
	    out.close();
	    in.close();
	} catch (IOException e) {
	    e.printStackTrace();
	    file.delete();
	    System.exit(1);
	}
    }

    /**
       SanctuaryXMLեɤޤ

       @param pm ѥåޥ͡
       @param xmlFile XMLե
    */
    private static void load(final PackageManager pm, final String xmlFile) {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        try {
	    DocumentBuilder builder = factory.newDocumentBuilder();
	    Document doc = builder.parse(new File(xmlFile));
	    Element root = doc.getDocumentElement();
	    if (root.getNodeName().equals("class")) {
		pm.addGroup(new Class((Element) root));
	    } else if (root.getNodeName().equals("interface")) {
		pm.addGroup(new Interface((Element) root));
	    } else if (root.getNodeName().equals("package")) {
		pm.addPackage(new Package((Element) root));
	    }
        } catch (SAXException sxe) {
	    Exception x = sxe;
	    if (sxe.getException() != null) {
		x = sxe.getException();
	    }
	    x.printStackTrace();
        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
        } catch (IOException ioe) {
	    ioe.printStackTrace();
        }
    }

    /**
       ˡɽޤ
    */
    private static void usage() {
	System.err.print(""
+ "Usage: java Sanctuary [options] [filename...]\n"
+ "Options are:\n"
+ "  --code[=TYPE] Generate header files.  TYPE can be \"c++\".\n"
+ "  --html        Generate HTML documents.\n"
+ "  --tex         Generate TeX documents.\n"
+ "  --stub        Generate stubs.\n"
+ "  --dtd         Generate DTD file (sanctuary.dtd).\n"
+ "");
	System.exit(1);
    }

    /**
       Sanctuary¹Ԥޤ

       @param av ޥɥ饤󥪥ץ
    */
    public static void main(final String[] av) {
	int k;
	boolean printCode = false;
	boolean printHtml = false;
	boolean printTex = false;
	boolean printStub = false;
	boolean printDtd = false;
	PackageManager pm = PackageManager.getInstance();
	Options opt = Options.getInstance();

	for (k = 0; k < av.length && av[k].startsWith("-"); ++k) {
	    if (av[k].equals("--html")) {
		printHtml = true;
	    } else if (av[k].equals("--code")) {
		printCode = true;
	    } else if (av[k].startsWith("--code=")) {
		printCode = true;
                int n = av[k].indexOf('=') + 1;
		opt.setCodeType(av[k].substring(n));
	    } else if (av[k].equals("--tex")) {
		printTex = true;
	    } else if (av[k].equals("--stub")) {
		printStub = true;
	    } else if (av[k].equals("--dtd")) {
		printDtd = true;
	    } else {
		usage();
	    }
	}
	if (printDtd) {
	    copyFile("sanctuary.dtd");
	}
	for (; k < av.length; ++k) {
	    load(pm, av[k]);
	}

	if (printHtml) {
	    pm.printIndexHtml();
	    copyFile("sanctuary.css");
	}
	if (printTex) {
	    pm.printSanctuaryTex();
	    copyFile("sanctuary.sty");
	}
	List<Group> v = pm.groupList();
	for (Group g : v) {
	    if (printCode) {
		g.printCode();
	    }
	    if (printHtml) {
		g.printHtml();
	    }
	    if (printTex) {
		g.printTex();
	    }
	    if (printStub) {
		g.printStub();
	    }
	}
    }
}
